'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true,
});
Object.defineProperty(exports, 'createSourceEventStream', {
  enumerable: true,
  get: function () {
    return _subscribe.createSourceEventStream;
  },
});
Object.defineProperty(exports, 'subscribe', {
  enumerable: true,
  get: function () {
    return _subscribe.subscribe;
  },
});

var _subscribe = require('../execution/subscribe.js');
