import type { ValueNode } from '../language/ast';
/**
 * Sort ValueNode.
 *
 * This function returns a sorted copy of the given ValueNode.
 *
 * @internal
 */
export declare function sortValueNode(valueNode: ValueNode): ValueNode;
