/**
 * Groups array items into a Map, given a function to produce grouping key.
 */
export declare function groupBy<K, T>(
  list: ReadonlyArray<T>,
  keyFn: (item: T) => K,
): Map<K, ReadonlyArray<T>>;
