/**
 * Upholds the spec rules about naming.
 */
export declare function assertName(name: string): string;
/**
 * Upholds the spec rules about naming enum values.
 *
 * @internal
 */
export declare function assertEnumValueName(name: string): string;
