import type { ASTVisitor } from '../../language/visitor';
import type { ASTValidationContext } from '../ValidationContext';
export declare function MaxIntrospectionDepthRule(
  context: ASTValidationContext,
): ASTVisitor;
