export interface ObjMap<T> {
  [key: string]: T;
}
export declare type ObjMapLike<T> =
  | ObjMap<T>
  | {
      [key: string]: T;
    };
export interface ReadOnlyObjMap<T> {
  readonly [key: string]: T;
}
export declare type ReadOnlyObjMapLike<T> =
  | ReadOnlyObjMap<T>
  | {
      readonly [key: string]: T;
    };
