/**
 * Given an invalid input string and a list of valid options, returns a filtered
 * list of valid options sorted based on their similarity with the input.
 */
export declare function suggestionList(
  input: string,
  options: ReadonlyArray<string>,
): Array<string>;
