/**
 * Memoizes the provided three-argument function.
 */
export declare function memoize3<
  A1 extends object,
  A2 extends object,
  A3 extends object,
  R,
>(fn: (a1: A1, a2: A2, a3: A3) => R): (a1: A1, a2: A2, a3: A3) => R;
